/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "merc.h"



/*
 * The following special functions are available for mobiles.
 */
DECLARE_SPEC_FUN( spec_breath_any );
DECLARE_SPEC_FUN( spec_breath_acid );
DECLARE_SPEC_FUN( spec_breath_fire );
DECLARE_SPEC_FUN( spec_breath_frost );
DECLARE_SPEC_FUN( spec_breath_gas );
DECLARE_SPEC_FUN( spec_breath_lightning );
DECLARE_SPEC_FUN( spec_cast_adept );
DECLARE_SPEC_FUN( spec_cast_cleric );
DECLARE_SPEC_FUN( spec_cast_judge );
DECLARE_SPEC_FUN( spec_cast_mage );
DECLARE_SPEC_FUN( spec_cast_undead );
DECLARE_SPEC_FUN( spec_executioner );
DECLARE_SPEC_FUN( spec_fido );
DECLARE_SPEC_FUN( spec_guard );
DECLARE_SPEC_FUN( spec_janitor );
DECLARE_SPEC_FUN( spec_mayor );
DECLARE_SPEC_FUN( spec_poison );
DECLARE_SPEC_FUN( spec_thief );



/*
 * Given a name, return the appropriate spec fun.
 */
SPEC_FUN *spec_lookup( const char *name )
{
   if( !str_cmp( name, "spec_breath_any" ) )
      return spec_breath_any;
   if( !str_cmp( name, "spec_breath_acid" ) )
      return spec_breath_acid;
   if( !str_cmp( name, "spec_breath_fire" ) )
      return spec_breath_fire;
   if( !str_cmp( name, "spec_breath_frost" ) )
      return spec_breath_frost;
   if( !str_cmp( name, "spec_breath_gas" ) )
      return spec_breath_gas;
   if( !str_cmp( name, "spec_breath_lightning" ) )
      return spec_breath_lightning;
   if( !str_cmp( name, "spec_cast_adept" ) )
      return spec_cast_adept;
   if( !str_cmp( name, "spec_cast_cleric" ) )
      return spec_cast_cleric;
   if( !str_cmp( name, "spec_cast_judge" ) )
      return spec_cast_judge;
   if( !str_cmp( name, "spec_cast_mage" ) )
      return spec_cast_mage;
   if( !str_cmp( name, "spec_cast_undead" ) )
      return spec_cast_undead;
   if( !str_cmp( name, "spec_executioner" ) )
      return spec_executioner;
   if( !str_cmp( name, "spec_fido" ) )
      return spec_fido;
   if( !str_cmp( name, "spec_guard" ) )
      return spec_guard;
   if( !str_cmp( name, "spec_janitor" ) )
      return spec_janitor;
   if( !str_cmp( name, "spec_mayor" ) )
      return spec_mayor;
   if( !str_cmp( name, "spec_poison" ) )
      return spec_poison;
   if( !str_cmp( name, "spec_thief" ) )
      return spec_thief;
   return 0;
}



/*
 * Core procedure for dragons.
 */
bool dragon( CHAR_DATA * ch, char *spell_name )
{
   CHAR_DATA *victim;
   CHAR_DATA *v_next;
   int sn;

   if( ch->position != POS_FIGHTING )
      return FALSE;

   for( victim = ch->in_room->people; victim != NULL; victim = v_next )
   {
      v_next = victim->next_in_room;
      if( victim->fighting == ch && number_bits( 2 ) == 0 )
         break;
   }

   if( victim == NULL )
      return FALSE;

   if( ( sn = skill_lookup( spell_name ) ) < 0 )
      return FALSE;
   ( *skill_table[sn].spell_fun ) ( sn, ch->level, ch, victim );
   return TRUE;
}



/*
 * Special procedures for mobiles.
 */
bool spec_breath_any( CHAR_DATA * ch )
{
   if( ch->position != POS_FIGHTING )
      return FALSE;

   switch ( number_bits( 3 ) )
   {
      case 0:
         return spec_breath_fire( ch );
      case 1:
      case 2:
         return spec_breath_lightning( ch );
      case 3:
         return spec_breath_gas( ch );
      case 4:
         return spec_breath_acid( ch );
      case 5:
      case 6:
      case 7:
         return spec_breath_frost( ch );
   }

   return FALSE;
}



bool spec_breath_acid( CHAR_DATA * ch )
{
   return dragon( ch, "acid breath" );
}



bool spec_breath_fire( CHAR_DATA * ch )
{
   return dragon( ch, "fire breath" );
}



bool spec_breath_frost( CHAR_DATA * ch )
{
   return dragon( ch, "frost breath" );
}



bool spec_breath_gas( CHAR_DATA * ch )
{
   int sn;

   if( ch->position != POS_FIGHTING )
      return FALSE;

   if( ( sn = skill_lookup( "gas breath" ) ) < 0 )
      return FALSE;
   ( *skill_table[sn].spell_fun ) ( sn, ch->level, ch, NULL );
   return TRUE;
}



bool spec_breath_lightning( CHAR_DATA * ch )
{
   return dragon( ch, "lightning breath" );
}



bool spec_cast_adept( CHAR_DATA * ch )
{
   CHAR_DATA *victim;
   CHAR_DATA *v_next;

   if( !IS_AWAKE( ch ) )
      return FALSE;

   for( victim = ch->in_room->people; victim != NULL; victim = v_next )
   {
      v_next = victim->next_in_room;
      if( victim != ch && can_see( ch, victim ) && number_bits( 1 ) == 0 )
         break;
   }

   if( victim == NULL )
      return FALSE;

   switch ( number_bits( 3 ) )
   {
      case 0:
         act( "$n utters the word 'tehctah'.", ch, NULL, NULL, TO_ROOM );
         spell_armor( skill_lookup( "armor" ), ch->level, ch, victim );
         return TRUE;

      case 1:
         act( "$n utters the word 'nhak'.", ch, NULL, NULL, TO_ROOM );
         spell_bless( skill_lookup( "bless" ), ch->level, ch, victim );
         return TRUE;

      case 2:
         act( "$n utters the word 'yeruf'.", ch, NULL, NULL, TO_ROOM );
         spell_cure_blindness( skill_lookup( "cure blindness" ), ch->level, ch, victim );
         return TRUE;

      case 3:
         act( "$n utters the word 'garf'.", ch, NULL, NULL, TO_ROOM );
         spell_cure_light( skill_lookup( "cure light" ), ch->level, ch, victim );
         return TRUE;

      case 4:
         act( "$n utters the words 'rozar'.", ch, NULL, NULL, TO_ROOM );
         spell_cure_poison( skill_lookup( "cure poison" ), ch->level, ch, victim );
         return TRUE;

      case 5:
         act( "$n utters the words 'nadroj'.", ch, NULL, NULL, TO_ROOM );
         spell_refresh( skill_lookup( "refresh" ), ch->level, ch, victim );
         return TRUE;

   }

   return FALSE;
}



bool spec_cast_cleric( CHAR_DATA * ch )
{
   CHAR_DATA *victim;
   CHAR_DATA *v_next;
   char *spell;
   int sn;

   if( ch->position != POS_FIGHTING )
      return FALSE;

   for( victim = ch->in_room->people; victim != NULL; victim = v_next )
   {
      v_next = victim->next_in_room;
      if( victim->fighting == ch && number_bits( 2 ) == 0 )
         break;
   }

   if( victim == NULL )
      return FALSE;

   for( ;; )
   {
      int min_level;

      switch ( number_bits( 4 ) )
      {
         case 0:
            min_level = 0;
            spell = "blindness";
            break;
         case 1:
            min_level = 3;
            spell = "cause serious";
            break;
         case 2:
            min_level = 7;
            spell = "earthquake";
            break;
         case 3:
            min_level = 9;
            spell = "cause critical";
            break;
         case 4:
            min_level = 10;
            spell = "dispel evil";
            break;
         case 5:
            min_level = 12;
            spell = "curse";
            break;
         case 6:
            min_level = 12;
            spell = "change sex";
            break;
         case 7:
            min_level = 13;
            spell = "flamestrike";
            break;
         case 8:
         case 9:
         case 10:
            min_level = 15;
            spell = "harm";
            break;
         default:
            min_level = 16;
            spell = "dispel magic";
            break;
      }

      if( ch->level >= min_level )
         break;
   }

   if( ( sn = skill_lookup( spell ) ) < 0 )
      return FALSE;
   ( *skill_table[sn].spell_fun ) ( sn, ch->level, ch, victim );
   return TRUE;
}



bool spec_cast_judge( CHAR_DATA * ch )
{
   CHAR_DATA *victim;
   CHAR_DATA *v_next;
   char *spell;
   int sn;

   if( ch->position != POS_FIGHTING )
      return FALSE;

   for( victim = ch->in_room->people; victim != NULL; victim = v_next )
   {
      v_next = victim->next_in_room;
      if( victim->fighting == ch && number_bits( 2 ) == 0 )
         break;
   }

   if( victim == NULL )
      return FALSE;

   spell = "high explosive";
   if( ( sn = skill_lookup( spell ) ) < 0 )
      return FALSE;
   ( *skill_table[sn].spell_fun ) ( sn, ch->level, ch, victim );
   return TRUE;
}



bool spec_cast_mage( CHAR_DATA * ch )
{
   CHAR_DATA *victim;
   CHAR_DATA *v_next;
   char *spell;
   int sn;

   if( ch->position != POS_FIGHTING )
      return FALSE;

   for( victim = ch->in_room->people; victim != NULL; victim = v_next )
   {
      v_next = victim->next_in_room;
      if( victim->fighting == ch && number_bits( 2 ) == 0 )
         break;
   }

   if( victim == NULL )
      return FALSE;

   for( ;; )
   {
      int min_level;

      switch ( number_bits( 4 ) )
      {
         case 0:
            min_level = 0;
            spell = "blindness";
            break;
         case 1:
            min_level = 3;
            spell = "chill touch";
            break;
         case 2:
            min_level = 7;
            spell = "weaken";
            break;
         case 3:
            min_level = 8;
            spell = "teleport";
            break;
         case 4:
            min_level = 11;
            spell = "colour spray";
            break;
         case 5:
            min_level = 12;
            spell = "change sex";
            break;
         case 6:
            min_level = 13;
            spell = "energy drain";
            break;
         case 7:
         case 8:
         case 9:
            min_level = 15;
            spell = "fireball";
            break;
         default:
            min_level = 20;
            spell = "acid blast";
            break;
      }

      if( ch->level >= min_level )
         break;
   }

   if( ( sn = skill_lookup( spell ) ) < 0 )
      return FALSE;
   ( *skill_table[sn].spell_fun ) ( sn, ch->level, ch, victim );
   return TRUE;
}



bool spec_cast_undead( CHAR_DATA * ch )
{
   CHAR_DATA *victim;
   CHAR_DATA *v_next;
   char *spell;
   int sn;

   if( ch->position != POS_FIGHTING )
      return FALSE;

   for( victim = ch->in_room->people; victim != NULL; victim = v_next )
   {
      v_next = victim->next_in_room;
      if( victim->fighting == ch && number_bits( 2 ) == 0 )
         break;
   }

   if( victim == NULL )
      return FALSE;

   for( ;; )
   {
      int min_level;

      switch ( number_bits( 4 ) )
      {
         case 0:
            min_level = 0;
            spell = "curse";
            break;
         case 1:
            min_level = 3;
            spell = "weaken";
            break;
         case 2:
            min_level = 6;
            spell = "chill touch";
            break;
         case 3:
            min_level = 9;
            spell = "blindness";
            break;
         case 4:
            min_level = 12;
            spell = "poison";
            break;
         case 5:
            min_level = 15;
            spell = "energy drain";
            break;
         case 6:
            min_level = 18;
            spell = "harm";
            break;
         case 7:
            min_level = 21;
            spell = "teleport";
            break;
         default:
            min_level = 24;
            spell = "gate";
            break;
      }

      if( ch->level >= min_level )
         break;
   }

   if( ( sn = skill_lookup( spell ) ) < 0 )
      return FALSE;
   ( *skill_table[sn].spell_fun ) ( sn, ch->level, ch, victim );
   return TRUE;
}



bool spec_executioner( CHAR_DATA * ch )
{
   char buf[MAX_STRING_LENGTH];
   CHAR_DATA *victim;
   CHAR_DATA *v_next;
   char *crime;

   if( !IS_AWAKE( ch ) || ch->fighting != NULL )
      return FALSE;

   crime = "";
   for( victim = ch->in_room->people; victim != NULL; victim = v_next )
   {
      v_next = victim->next_in_room;

      if( !IS_NPC( victim ) && IS_SET( victim->act, PLR_KILLER ) )
      {
         crime = "KILLER";
         break;
      }

      if( !IS_NPC( victim ) && IS_SET( victim->act, PLR_THIEF ) )
      {
         crime = "THIEF";
         break;
      }
   }

   if( victim == NULL )
      return FALSE;

   sprintf( buf, "%s is a %s!  PROTECT THE INNOCENT!  MORE BLOOOOD!!!", victim->name, crime );
   do_shout( ch, buf );
   multi_hit( ch, victim, TYPE_UNDEFINED );
   char_to_room( create_mobile( get_mob_index( MOB_VNUM_CITYGUARD ) ), ch->in_room );
   char_to_room( create_mobile( get_mob_index( MOB_VNUM_CITYGUARD ) ), ch->in_room );
   return TRUE;
}



bool spec_fido( CHAR_DATA * ch )
{
   OBJ_DATA *corpse;
   OBJ_DATA *c_next;
   OBJ_DATA *obj;
   OBJ_DATA *obj_next;

   if( !IS_AWAKE( ch ) )
      return FALSE;

   for( corpse = ch->in_room->contents; corpse != NULL; corpse = c_next )
   {
      c_next = corpse->next_content;
      if( corpse->item_type != ITEM_CORPSE_NPC )
         continue;

      act( "$n savagely devours a corpse.", ch, NULL, NULL, TO_ROOM );
      for( obj = corpse->contains; obj; obj = obj_next )
      {
         obj_next = obj->next_content;
         obj_from_obj( obj );
         obj_to_room( obj, ch->in_room );
      }
      extract_obj( corpse );
      return TRUE;
   }

   return FALSE;
}



bool spec_guard( CHAR_DATA * ch )
{
   char buf[MAX_STRING_LENGTH];
   CHAR_DATA *victim;
   CHAR_DATA *v_next;
   CHAR_DATA *ech;
   char *crime;
   int max_evil;

   if( !IS_AWAKE( ch ) || ch->fighting != NULL )
      return FALSE;

   max_evil = 300;
   ech = NULL;
   crime = "";

   for( victim = ch->in_room->people; victim != NULL; victim = v_next )
   {
      v_next = victim->next_in_room;

      if( !IS_NPC( victim ) && IS_SET( victim->act, PLR_KILLER ) )
      {
         crime = "KILLER";
         break;
      }

      if( !IS_NPC( victim ) && IS_SET( victim->act, PLR_THIEF ) )
      {
         crime = "THIEF";
         break;
      }

      if( victim->fighting != NULL && victim->fighting != ch && victim->alignment < max_evil )
      {
         max_evil = victim->alignment;
         ech = victim;
      }
   }

   if( victim != NULL )
   {
      sprintf( buf, "%s is a %s!  PROTECT THE INNOCENT!!  BANZAI!!", victim->name, crime );
      do_shout( ch, buf );
      multi_hit( ch, victim, TYPE_UNDEFINED );
      return TRUE;
   }

   if( ech != NULL )
   {
      act( "$n screams 'PROTECT THE INNOCENT!!  BANZAI!!", ch, NULL, NULL, TO_ROOM );
      multi_hit( ch, ech, TYPE_UNDEFINED );
      return TRUE;
   }

   return FALSE;
}



bool spec_janitor( CHAR_DATA * ch )
{
   OBJ_DATA *trash;
   OBJ_DATA *trash_next;

   if( !IS_AWAKE( ch ) )
      return FALSE;

   for( trash = ch->in_room->contents; trash != NULL; trash = trash_next )
   {
      trash_next = trash->next_content;
      if( !IS_SET( trash->wear_flags, ITEM_TAKE ) )
         continue;
      if( trash->item_type == ITEM_DRINK_CON || trash->item_type == ITEM_TRASH || trash->cost < 10 )
      {
         act( "$n picks up some trash.", ch, NULL, NULL, TO_ROOM );
         obj_from_room( trash );
         obj_to_char( trash, ch );
         return TRUE;
      }
   }

   return FALSE;
}



bool spec_mayor( CHAR_DATA * ch )
{
   static const char open_path[] = "W3a3003b33000c111d0d111Oe333333Oe22c222112212111a1S.";

   static const char close_path[] = "W3a3003b33000c111d0d111CE333333CE22c222112212111a1S.";

   static const char *path;
   static int pos;
   static bool move;

   if( !move )
   {
      if( time_info.hour == 6 )
      {
         path = open_path;
         move = TRUE;
         pos = 0;
      }

      if( time_info.hour == 20 )
      {
         path = close_path;
         move = TRUE;
         pos = 0;
      }
   }

   if( ch->fighting != NULL )
      return spec_cast_cleric( ch );
   if( !move || ch->position < POS_SLEEPING )
      return FALSE;

   switch ( path[pos] )
   {
      case '0':
      case '1':
      case '2':
      case '3':
         move_char( ch, path[pos] - '0' );
         break;

      case 'W':
         ch->position = POS_STANDING;
         act( "$n awakens and groans loudly.", ch, NULL, NULL, TO_ROOM );
         break;

      case 'S':
         ch->position = POS_SLEEPING;
         act( "$n lies down and falls asleep.", ch, NULL, NULL, TO_ROOM );
         break;

      case 'a':
         act( "$n says 'Hello Honey!'", ch, NULL, NULL, TO_ROOM );
         break;

      case 'b':
         act( "$n says 'What a view!  I must do something about that dump!'", ch, NULL, NULL, TO_ROOM );
         break;

      case 'c':
         act( "$n says 'Vandals!  Youngsters have no respect for anything!'", ch, NULL, NULL, TO_ROOM );
         break;

      case 'd':
         act( "$n says 'Good day, citizens!'", ch, NULL, NULL, TO_ROOM );
         break;

      case 'e':
         act( "$n says 'I hereby declare the city of Midgaard open!'", ch, NULL, NULL, TO_ROOM );
         break;

      case 'E':
         act( "$n says 'I hereby declare the city of Midgaard closed!'", ch, NULL, NULL, TO_ROOM );
         break;

      case 'O':
         do_unlock( ch, "gate" );
         do_open( ch, "gate" );
         break;

      case 'C':
         do_close( ch, "gate" );
         do_lock( ch, "gate" );
         break;

      case '.':
         move = FALSE;
         break;
   }

   pos++;
   return FALSE;
}



bool spec_poison( CHAR_DATA * ch )
{
   CHAR_DATA *victim;

   if( ch->position != POS_FIGHTING || ( victim = ch->fighting ) == NULL || number_percent(  ) > 2 * ch->level )
      return FALSE;

   act( "You bite $N!", ch, NULL, victim, TO_CHAR );
   act( "$n bites $N!", ch, NULL, victim, TO_NOTVICT );
   act( "$n bites you!", ch, NULL, victim, TO_VICT );
   spell_poison( gsn_poison, ch->level, ch, victim );
   return TRUE;
}



bool spec_thief( CHAR_DATA * ch )
{
   CHAR_DATA *victim;
   CHAR_DATA *v_next;
   int gold;

   if( ch->position != POS_STANDING )
      return FALSE;

   for( victim = ch->in_room->people; victim != NULL; victim = v_next )
   {
      v_next = victim->next_in_room;

      if( IS_NPC( victim ) || victim->level >= LEVEL_IMMORTAL || number_bits( 2 ) != 0 || !can_see( ch, victim ) )   /* Thx Glop */
         continue;

      if( IS_AWAKE( victim ) && number_range( 0, ch->level ) == 0 )
      {
         act( "You discover $n's hands in your wallet!", ch, NULL, victim, TO_VICT );
         act( "$N discovers $n's hands in $S wallet!", ch, NULL, victim, TO_NOTVICT );
         return TRUE;
      }
      else
      {
         gold = victim->gold * number_range( 1, 20 ) / 100;
         ch->gold += 7 * gold / 8;
         victim->gold -= gold;
         return TRUE;
      }
   }

   return FALSE;
}
